@@ -18,6 +18,7 @@ probe_devices = [
 'sd[a-z][1-9]',
 'fd[0-1]',
 'cdrom',
+'sr[0-9]'
 ]
 
 blacklist_devices = [
@@ -51,7 +52,7 @@ mntdir = '/media/'
 
 # end of default options
 
-VERSION='0.4'
+VERSION='0.6'
 
 if not os.path.isfile(config_file):
     print "Missing configuration file", config_file
@@ -154,13 +155,18 @@ def safe_rmdir(path):
             os.rmdir(path)
     else:
         raise IOError, "Trying to remove suspicious directory: "+`path`
-    
+
 def get_all_devices(list_to_probe):
     r = []
     for i in list_to_probe:
         path = os.path.join(device_prefix, i)
+        path = os.path.normpath(path)
         for j in glob.glob(path):
-            r.append(j)
+            x = os.path.normpath(j)
+            if x.startswith(device_prefix):
+                r.append(j)
+            else:
+                alert("Discarding suspicious device "+`j`)
     return r
 
 def check_device(device):
@@ -174,6 +180,10 @@ def check_device(device):
 def try_mount(device, fs):
     dir_was_created = False
     mountpoint = os.path.join(mntdir, os.path.split(device)[1])
+    if os.path.islink(mountpoint) and not os.path.exists(mountpoint):
+        # broken symlink
+        info(mountpoint + " is a broken symlink - removing")
+        safe_rmdir(mountpoint)
     if os.path.exists(mountpoint):
         verbose(mountpoint + " already exists")
         if os.path.ismount(mountpoint):
@@ -197,12 +207,17 @@ def try_mount(device, fs):
 def try_umount(device):
     mountpoint = os.path.join(mntdir, os.path.split(device)[1])
     if os.path.islink(mountpoint):
-        verbose(mountpoint + " is a symlink - removing")
-        safe_rmdir(mountpoint)
+        #verbose(mountpoint + " is a symlink - removing")
+        #safe_rmdir(mountpoint)
+        verbose(mountpoint + " is a symlink - skipping")
         return None
     if not os.path.exists(mountpoint):
         verbose(mountpoint + " does not exists")
         return None
+    if not os.path.ismount(mountpoint):
+        verbose(mountpoint + " is not mountpoint")
+        return None
+        
 #    if not os.path.ismount(mountpoint):
 #        verbose(mountpoint + " is not mounted - removing")
 #        safe_rmdir(mountpoint)
@@ -216,6 +231,31 @@ def try_umount(device):
         alert("Could not umount %s, reason: %s\n" %(device, output))
         return False # could not umount
 
+how_to_run = '''
+mountpy is not being run with enough privileges.
+There are 3 ways how to make mountpy run comfortably with root privileges:
+1) install sudo, add this line to /etc/sudoers:
+joesmith ALL = NOPASSWD: /usr/bin/mountpy, usr/bin/umountpy
+   to allow user joesmith use mountpy.
+   It you want to allow all the users use mountpy, change the line into:
+ ALL ALL = NOPASSWD: /usr/bin/mountpy, usr/bin/umountpy
+   and then run "sudo mountpy" and "sudo umountpy"
+2) install super, add these lines to /etc/super.tab:
+   mountpy /usr/bin/mountpy.py joesmith
+umountpy /usr/bin/umountpy joesmith
+   to allow user joesmith to run mountpy, or:
+mountpy /usr/bin/mountpy.py .*
+umountpy /usr/bin/umountpy .*
+   to give access to all the users, and run "super mountpy" or "super umountpy"
+3) make the C wrapper script setuid root
+   log in as root and type:
+chmod u+s /usr/bin/mountpy
+
+Be aware that giving all the users rights to execute mountpy can be a
+security risk, if an exploitable bug is found in mountpy.  This is
+especially true in the 3rd case. Therefore, it is recommended to allow
+only selected users to run mountpy, and use super or sudo if possible.
+'''
     
 def verbose(text, verbosity = 1):
     if options.verbosity >= verbosity:
@@ -263,10 +303,17 @@ uid = os.getuid()
 gid = os.getgid()
 euid = os.geteuid()
 
+# test if we are running with sudo
+if uid==0:
+    if os.environ.has_key('SUDO_UID'):
+        uid = int(os.environ['SUDO_UID'])
+        verbose("sudo detected")
+
 verbose("Detected UID %s, GID %s, EUID %s" % (uid, gid, euid))
 
 if euid != 0:
-    info("Not running setuid root, but trying anyway")
+    msg(how_to_run)
+    raw_input("Press ENTER to try anyway")
 
 os.setuid(0)
 
